
#if !defined(TYYBID_H)
#define TYYBID_H

#include "suurused.h"

    #if !defined(_int8)
        #define _int8   signed char 
        #define _int16  signed short
        #define _int32  signed int
        #define _uint8  unsigned char
        #define _uint16 unsigned short
        #define _uint32 unsigned int
    #endif
    #define _float32 float

#endif


